﻿const webpack = require('webpack');
const path = require('path');
const fs = require('fs');
let utils = light.util;
let shell = utils.require('shelljs');

exports.build = function () {

    return Promise.all([
        //JSNative
        new Promise(function (resolve) {
            if(require("./lib/utils").hasPlugin("jsnative") && (!light.options.onlyFor || light.options.onlyFor === 'jsn')){
                let config_native = require("./build.native").build();
                return webpack(config_native, function () {
                    resolve();
                })
            }else{
                resolve()
            }
        }),
        //miniapp
        new Promise(function (resolve) {
            if(require("./lib/utils").hasPlugin("miniapp") && (!light.options.onlyFor || light.options.onlyFor === 'miniapp')){
                let config_native = require("./build.miniapp").build();
                // 【小程序】小程序在编译构建时支持热刷新功能，当开发者编辑代码后大约5s左右触发手机端程序界面的自动加载刷新
                global.middleware = global.middleware || [];
                let clinetCheckMap = {};
                global.middleware.push(function(req,res,next){
                    
                    if(req.path === "/check_reload_me"){
                        if(!clinetCheckMap[req.query.id]){
                            clinetCheckMap[req.query.id] = false;
                        }
                        res.send({
                            reload:clinetCheckMap[req.query.id]
                        })
                        clinetCheckMap[req.query.id] = false;
                    }else{
                        next()
                    }
                });
                return webpack(config_native, function () {
                    Object.keys(clinetCheckMap).forEach(client=>{clinetCheckMap[client]=true});
                    resolve();
                })
            }else{
                resolve()
            }
        }),
        //web
        new Promise(function (resolve) {
            //110579：若--only-for jsn，则不编译h5
            if (light.options.onlyFor &&  light.options.onlyFor !== 'h5') {
               return resolve()
            }

            let config_web = require("./build.web").build();
            if (process.env.NODE_ENV === "dev") {
                const compiler = webpack(config_web);
                //122625 【TS:201902110033-财富业委会-潘腾-【需求类型】	需求【需求描述】	1. 前后端分离开发】
                global.middleware = global.middleware || [];

                const devModule = require("webpack-dev-middleware")(compiler, {
                    noInfo: true, log: false
                });
                global.middleware.push(devModule);

                devModule.waitUntilValid(() => {
                    resolve()
                });

                global.middleware.push(require("webpack-hot-middleware")(compiler, {
                    noInfo: true, log: false
                }))

                //97319 指定输出的编译文件目录
                //175283 支持 light release -w -o 指定编译目录
            } else {
                webpack(config_web, function () {
                    resolve();
                });
            }
        }),
        //230279 light-sdk支持全局JSCore
        new Promise(function (resolve) {
            let gmuGlobalJS = require("path").join(light.config.src, "gmu.global.js");
            if (require("fs").existsSync(gmuGlobalJS)) {
                let config_globalJscore = require("./build.globalJscore").build();
                return webpack(config_globalJscore, function () {
                    resolve();
                })
            } else {
                return resolve()
            }
        })
    ])
    //175283 支持 light release -w -o 指定编译目录
};
