let utils = light.util;
let config = light.config;
let shell = utils.require('shelljs');
let version = require('./package.json').version;

let vueTemplate = utils.template(`
/**AUTO-GEN-START*/
import __VUE__ from "vue"
__VUE__.use(__LIGHT__);
import __LIGHT__ from "light"
__LIGHT__.mainPage = '<%=mainPage%>'
__LIGHT__.onlyFor = '<%=onlyFor%>'
__LIGHT__
<%views.forEach(function(view){%>
.route({
    path: '/<%=view.id%>',
    <%if(view.async=="true"){%>component: ()=>{return import(/* webpackChunkName: "<%=view.chunkname||page+'-'+view.name%>" */"./<%=view.id%>.vue")},<%}%>
    <%if(!view.async||view.async=="false"){%>component: require("./<%=view.id%>.vue"),<%}%>
    <%if(view.parent){%>parent:"/<%=view.parent%>",<%}%>
    <%if(view.home){%>home:"/<%=view.home%>",<%}%>
    <%for(var key in view){%><%if(['id', 'async', 'parent', 'home', 'chunkname'].indexOf(key) == -1){%><%=key%>: "<%=view[key]%>",<%}%><%}%>
})
<%})%>
/**AUTO-GEN-END*/
`);



module.exports = {
    pluginName: `type-vue@${version}`,
    install: function (light) {
    },

    /**
     * 准备阶段
     */
    prepare: function (light, cb) {
        // 修复在lighting工程中执行light searver --debug 时无法正常执行的问题
        if (!utils.existsSync(config.dist)) return cb();

        var Module = require('module');
        var originalRequire = Module.prototype.require;

        Module.prototype.require = function (m) {
            //do your thing here
            if (m === "./JsonpMainTemplatePlugin") {
                return require("./plugins/JsonpMainTemplatePlugin.js")
            }
            return originalRequire.apply(this, arguments);
        };

        //准备编译
        let promise;

        //1. 自定义标签的处理
        //1.1 view标签的处理

        let works = [];
        utils.glob(`${config.dist}/*.html`, function (err, files) {
            //105239 多页工程支持分页面打包
            let fs = require('fs');
            if (light.options.page && fs.existsSync(`${config.dist}/${light.options.page}.html`)) {
                files = [`${config.dist}/${light.options.page}.html`]
            }
            files.forEach(function (file) {
                let viewsArr = [];
                works.push(new Promise(function (resolve, reject) {
                    let temparr = file.split('/');
                    let filename = temparr[temparr.length - 1];
                    filename = filename.replace('.html', '');
                    let $ = utils.require("cheerio").load(utils.readFileSync(file), {
                        recognizeSelfClosing: true
                    });

                    let views = $("view");
                    views.first().replaceWith("<router-view></router-view>");
                    views.remove();

                    utils.outputFile(file, $.html(), (err) => {
                        viewsArr = {
                            name: filename,
                            views: views
                        }
                        resolve(viewsArr);
                    });

                }))
            })

            Promise.all(works).then(function (viewsArr) {

                //1.2 根据视图内容生成视图注册的模板，注册路由
                //105231 确定主页名称为app或index并注入Light
                var mainPage = require('./plugins/entryFile').getEntryFile().mainPage;

                viewsArr.forEach(function (viewObj) {
                    let viewArr = viewObj.views;
                    let v = [];
                    viewArr.each(function (i, view) {
                        let id = view.attribs.id;
                        view.attribs.name = id.replace(/\//g, '-')
                        v.push(view.attribs)
                    });
                    //110579：将onlyFor注入Light
                    let vueViewString = vueTemplate({ views: v, page: viewObj.name, mainPage: mainPage, onlyFor: light.options.onlyFor });
                    utils.writeFileSync(`${config.dist}/view/${viewObj.name}.js`, `${vueViewString}`);

                })

                cb();

            });
        })

    },

    /**
     * 编译阶段
     */
    build(light, cb) {
        //2 编译，通过webpack把所有资源整合起来
        new Promise(function (resolve, reject) {
            process.chdir(config.dist);
            process.mainModule.paths.push(`${__dirname}/node_modules`);

            //环境变量
            process.env.NODE_ENV = process.env.NODE_ENV || "production";
            return require("./build").build().then(resolve);
        }).then(cb);
    },

    /**
     * 打包阶段
     */
    packager(light, cb) {
        shell.rm("-fr", `${config.dist}/view`);
        shell.rm("-fr", `${config.dist}/ui`);
        shell.rm("-fr", `${config.dist}/html`);
        shell.rm("-fr", `${config.dist}/js`);
        shell.rm("-fr", `${config.dist}/lib`);

        //145355 打包时移除根目录的app.less
        shell.rm("-fr", `${config.dist}/app.less`);
        shell.rm("-fr", `${config.dist}/api`);
        shell.rm("-fr", `${config.dist}/plugins`);
        //114521 【工具】工程根目录中有node_modules目录时移除它
        shell.rm("-fr",`${config.dist}/node_modules`);
        //118795 lighting的native插件打包时-p，此时删除目录中存在的modules目录
        shell.rm("-fr",`${config.dist}/modules`);
        //116125【TS:201812210068-财富业委会-潘腾-【需求类型】需求【需求描述】1. light release -p或者-up时，能增加参数支持 不要将某些文件夹打入zip包
        if(light.options.except){
            var exceptFolders = light.options.except.split(",");
            exceptFolders.forEach(function(folder){
                shell.rm("-fr", `${config.dist}/${folder}`);
            });
        }


        //184657【type-vue插件】打包阶段将框架信息写入project.json文件
        //189105【type-vue插件】修复macOS上打包报错的问题
        writeProjectInfo();
        function writeProjectInfo() {
            if(utils.getLightingVersion) {
                var path = require('path'), fs = require('fs');
                light.config.lightToolkits = {
                    lighting: utils.getLightingVersion(),
                    plugins : []
                }
            
                if(!config.plugins) config['plugins'] = ['type-vue']
                var plugins = config.plugins;

                plugins && plugins.forEach((plugin)=>{
                    let pluginConfigDir = utils.join(config.pluginsDir, `node_modules/lighting-plugin-${plugin}/package.json`);
                    let pluginVersion = require(pluginConfigDir).version;
                    plugin = plugin.replace('@'+pluginVersion, '');
                    light.config.lightToolkits.plugins.push(`${plugin}@${pluginVersion}`) 
                })
                
                let projectConfigFile  = path.join(config.dist,"project.json");
                if(utils.existsSync(projectConfigFile)){
                    let projectConfig = require(projectConfigFile);
                    projectConfig['lightToolkits'] = light.config.lightToolkits;
                    fs.writeFileSync(projectConfigFile, JSON.stringify(projectConfig,null,"\t"))
                }
            }
        }

        cb();
    },

    /**
     * 运行阶段
     */
    runtime: function () {

    }
};