let webpack = require('webpack');

process.traceDeprecation = false;
process.noDeprecation = true;

var pages = require('./plugins/entryFile').getEntryFile().pages;

const babelOptions = {
    presets: [
        require('babel-preset-env'),
        require('babel-preset-stage-0'),
    ],
    plugins: [
        // require('babel-plugin-transform-runtime'),
        require('babel-plugin-syntax-dynamic-import'),
        require("babel-plugin-transform-class-properties"),
        //	lighting支持jsx编译
        require('babel-plugin-syntax-jsx'),
        require('babel-plugin-transform-vue-jsx')
    ],
    compact: 'auto',
    //73460 light工程中lib目录中存在node_modules目录中的ethjs-util模块时编译不报错,打包报错的问题处理
    babelrc:false,
    //94259 lighting中编译支持缓存，提高编译效率
    // cacheDirectory:require("path").join(require("os").tmpdir(),'LIGHTING_WEBPACK_CACHE')
};



// 91743 多页工程
var entryArr = {};
if (pages && pages.length > 0) {
    pages.forEach(function (page) {
        //105231 指定多页工程的入口文件
        let entry = {};
        let key = pages.length == 1 && pages[0] == 'index' && light.options.page != 'index' ? 'app' : page;
        let mainjs = page == 'index' ? 'app' : page;
        entry[key] = [`${light.config.dist}/view/${page}.js`];

        //118407 lighting支持当light工程不存在app.js时的打包
        const appFile = `${light.config.dist}/${mainjs}.js`;
        if(light.util.existsSync(appFile)){
            entry[key].push(appFile);
        }
        entryArr = Object.assign(entryArr, entry)
    })
}
//bug94269 缓存时图片不添加时间戳后缀
let imghash = light.config.plugins && light.config.plugins.toString().indexOf('cachemanifest') > -1;

let config = {
    entry: entryArr,
    module: {
        rules: [
            {
                test: /\.(less|css)$/,
                use: [{
                    loader: "style-loader" // creates style nodes from JS strings
                }, {
                    loader: "css-loader" // translates CSS into CommonJS
                }, {
                    loader: "less-loader" // compiles Less to CSS
                }]
            },
            {
                test: /\.js$/,
                //去除node_modules目录的es6转es5流程，加快编译速度
                exclude: process.env.NODE_ENV === "dev" ? [function (file) {
                    return /(node_modules\/[^(olight)]|bower_components)/ig.test(file)
                }] : undefined,
                loader: 'babel-loader',
                options:babelOptions

            },
            {
                test: /\.(png|jpg|gif|svg|ttf|woff|eot|otf)$/,
                loader: 'file-loader',
                options: {
                    name: imghash ? 'lightdist/[name].[ext]' : 'lightdist/[name].[ext]?[hash]'
                }
            }
        ]
    },
    resolve: {
        extensions: ['.js', '.vue', '.json'],
        modules: [`${__dirname}/node_modules`, "lib/node_modules", "lib", "node_modules",`${__dirname}/..`],
        alias: {
            "light": `olight`,
            'vue': "vue/dist/vue.esm",
            "@":light.config.dist
        }

    },
    plugins: [
        new require('progress-bar-webpack-plugin')({
            format: '  编译中 [:bar] ' + require("chalk").green.bold(':percent') + ' (:elapsed 秒)',
            clear: false
        }),
        new webpack.NoEmitOnErrorsPlugin(),
        new (require("./plugins/loader"))({
            babel: babelOptions
        })
    ],
    resolveLoader: {
        modules: [`${__dirname}/node_modules`, "lib/node_modules", "lib", "node_modules",`${__dirname}/..`]
    }
};

//102711 vue代码中有错误，比如import写错，使用light release -p 打包，无法报错
//227597 开发阶段报错提示
// if(!light.options.watch){
    config.plugins.push(function(){
        this.plugin("done", function(stats)
        {
            if (stats.compilation.errors && stats.compilation.errors.length)
            {
                stats.compilation.errors.forEach((err)=>{
                    if(err.error&&err.error['snippet']){
                        if(err.module&&err.module['issuer']&&err.module.issuer['rawRequest']) console.log('Error in: '.red + err.module.issuer.rawRequest.red)
                        console.log(err.error.snippet)
                    } 
                })
                console.log(stats.compilation.errors.toString().red)
            }
        });
    })
// }

if (light.options.uglify) {
    config = require("webpack-merge")({
        plugins: [
            new webpack.optimize.UglifyJsPlugin({
                compress: {
                    warnings: false
                }
            })
        ]
    }, config)
}

module.exports = config;
